# STA 410, Fall 2014, Assignment #2, First problem.


# Compute the log likelihood from rounded data modelled as i.i.d. normal, using
# R's integrate function to compute probabilities.

rnd_norm_loglik1 <- function (x, d, mu, sigma)
{
    ll <- 0
    e <- 0.5 * 10^-d
    for (i in 1:length(x))
        ll <- ll + log (integrate (function (v) dnorm(x[i]+v,mu,sigma), -e, e)
                         $ value)
    ll
}


# Find MLE for rounded i.i.d. data from a normal distribution, using R's
# integrate function to compute probabilities.

rnd_norm_mle1 <- function (x, d)
{
    r <- nlm (function (theta) -rnd_norm_loglik1(x,d,theta[1],exp(theta[2])),
              c (mean(x), log(sd(x))))

    c (mu=r$estimate[1], log_sigma=r$estimate[2], sigma=exp(r$estimate[2]))
}


# Compute the log likelihood from rounded data modelled as i.i.d. normal, using
# R's pnorm function to compute probabilities.

rnd_norm_loglik2 <- function (x, d, mu, sigma)
{
    ll <- 0
    e <- 0.5 * 10^-d
    for (i in 1:length(x))
        ll <- ll + log (pnorm (x[i]+e, mu, sigma) - pnorm (x[i]-e, mu, sigma))
    ll
}


# Find MLE for rounded i.i.d. data from a normal distribution, using R's
# pnorm function to compute probabilities.

rnd_norm_mle2 <- function (x, d)
{
    r <- nlm (function (theta) -rnd_norm_loglik2(x,d,theta[1],exp(theta[2])),
              c (mean(x), log(sd(x))))

    c (mu=r$estimate[1], log_sigma=r$estimate[2], sigma=exp(r$estimate[2]))
}


# Apply these functions to the three data sets.

cat("\nDATA SET 1:\n\n")
set.seed(1); d1 <- 1; x1 <- round(rnorm(25,2.24,1),d1)

print(system.time(mle1 <- rnd_norm_mle1(x1,d1)))
print(system.time(mle2 <- rnd_norm_mle2(x1,d1)))

print(rbind(mle1=mle1, mle2=mle2,
            sample=c(mu=mean(x1),log_sigma=log(sd(x1)),sigma=sd(x1)),
            true=c(mu=2.24,log_sigma=log(1),sigma=1)))

cat("\nDATA SET 2:\n\n")
set.seed(2); d2 <- 1; x2 <- round(rnorm(50,2.24,0.1),d2)

print(system.time(mle1 <- rnd_norm_mle1(x2,d2)))
print(system.time(mle2 <- rnd_norm_mle2(x2,d2)))

print(rbind(mle1=mle1, mle2=mle2,
            sample=c(mu=mean(x2),log_sigma=log(sd(x2)),sigma=sd(x2)),
            true=c(mu=2.24,log_sigma=log(0.1),sigma=0.1)))

cat("\nDATA SET 3:\n\n")
set.seed(3); d3 <- 0; x3 <- round(rnorm(4000,2.24,0.8),d3)

print(system.time(mle1 <- rnd_norm_mle1(x3,d3)))
print(system.time(mle2 <- rnd_norm_mle2(x3,d3)))

print(rbind(mle1=mle1, mle2=mle2,
            sample=c(mu=mean(x3),log_sigma=log(sd(x3)),sigma=sd(x3)),
            true=c(mu=2.24,log_sigma=log(0.8),sigma=0.8)))

cat("\nMORE RUNS WITH VARYING MEAN:\n\n")

for (m in seq(2,2.5,by=0.05)) {
    set.seed(3); d <- 0; x <- round(rnorm(4000,m,0.8),d)
    mle2 <- rnd_norm_mle2(x,d)
    print(rbind(mle2=mle2,
                sample=c(mu=mean(x),log_sigma=log(sd(x)),sigma=sd(x)),
                true=c(mu=m,log_sigma=log(0.8),sigma=0.8)))
}
